//
//  GFaAppSettingViewController.m
//
//  Copyright (c) 2022 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaAppSettingViewController.h"

#import "GFaAppSettingModel.h"
#import "GFaUtility.h"

/* セクション定数定義 */
enum {
    Section_TokenServer = 0,    /* トークン発行サーバ */
    Section_AreaServer          /* エリア情報サーバ */
};

/* フィールド定数定義(トークン発行サーバセクション) */
enum {
    Field_ClientId = 0,         /* クライアントID */
    Field_Secret,               /* 秘密鍵 */
    Field_TokenUrl              /* トークン発行サーバURL */
};

/* フィールド定数定義(エリア情報サーバセクション) */
enum {
    Field_ApiKey = 0,           /* APIキー */
    Field_searchAreaUpdTimeUrl, /* エリア更新日時検索URL */
    Field_searchAreaUrl         /* エリア検索URL */
};

@interface GFaAppSettingViewController ()

- (UITextField *)findTextField_:(NSInteger)section row:(NSInteger)row;
- (void)setValueToAppSetting_;
- (void)setValueFromAppSetting_;

@end

@implementation GFaAppSettingViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.title = @"アプリ設定";

    [self setValueFromAppSetting_];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    [self setValueToAppSetting_];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/* 初期化ボタン押下 */
- (IBAction)push_Default:(id)sender
{
    GFaAppSettingModel *model = [GFaAppSettingModel defaultModel];
    
    [model resetSetting];
    [model saveToUserDefaults];
    [self setValueFromAppSetting_];
}

#pragma mark UITableViewDelegate

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];

    __block UITextField *textField = [self findTextField_:indexPath.section row:indexPath.row];

    [GFaUtility showInputAlert:self title:textField.accessibilityLabel message:@"" text:textField.text cancelButtonTitle:@"キャンセル" otherButtonTitle:@"確定" keyboardType:textField.keyboardType otherHandler:^(NSString *value) {
        textField.text = value;
    }];
}

#pragma mark Private

/* 入力フィールド取得 */
- (UITextField *)findTextField_:(NSInteger)section row:(NSInteger)row
{
    switch (section) {
        case Section_TokenServer:
            switch (row) {
                case Field_ClientId:
                    return self.clientidTextField;
                case Field_Secret:
                    return self.secretTextField;
                case Field_TokenUrl:
                    return self.tokenUrlTextField;
                default:
                    break;
            }
        case Section_AreaServer:
            switch (row) {
                case Field_ApiKey:
                    return self.apikeyTextField;
                case Field_searchAreaUpdTimeUrl:
                    return self.searchAreaUpdTimeUrlTextField;
                case Field_searchAreaUrl:
                    return self.searchAreaUrlTextField;
                default:
                    break;
            }
        default:
            break;
    }
    return nil;
}

/* 画面入力値を設定保持クラスに設定 */
- (void)setValueToAppSetting_
{
    GFaAppSettingModel *settingModel = [GFaAppSettingModel defaultModel];
    [settingModel setObject:self.clientidTextField.text forKey:kAppKeyClientId];
    [settingModel setObject:self.secretTextField.text forKey:kAppKeySecret];
    [settingModel setObject:self.tokenUrlTextField.text forKey:kAppKeyTokenURL];
    [settingModel setObject:self.apikeyTextField.text forKey:kAppKeyApiKey];
    [settingModel setObject:self.searchAreaUpdTimeUrlTextField.text forKey:kAppKeySearchAreaUpdateTimeURL];
    [settingModel setObject:self.searchAreaUrlTextField.text forKey:kAppKeySearchAreaURL];

    [settingModel saveToUserDefaults];
}

/* 設定保持クラスの内容を画面入力値に設定 */
- (void)setValueFromAppSetting_
{
    GFaAppSettingModel *settingModel = [GFaAppSettingModel defaultModel];
    self.clientidTextField.text = [settingModel objectForKey:kAppKeyClientId];
    self.secretTextField.text = [settingModel objectForKey:kAppKeySecret];
    self.tokenUrlTextField.text = [settingModel objectForKey:kAppKeyTokenURL];
    self.apikeyTextField.text = [settingModel objectForKey:kAppKeyApiKey];
    self.searchAreaUpdTimeUrlTextField.text = [settingModel objectForKey:kAppKeySearchAreaUpdateTimeURL];
    self.searchAreaUrlTextField.text = [settingModel objectForKey:kAppKeySearchAreaURL];
}

@end
