//
//  GFaLocationManagerSettingViewController.m
//
//  Copyright (c) 2013 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "GFaLocationManagerSettingViewController.h"

#import <libLocationManagerProbe/LocationManager.h>
#import "GFaLocationManagerSettingModel.h"
#import "GFaAppDelegate.h"
#import "GFaUtility.h"
#import "GFaCommon.h"

/* セクション定数定義 */
enum {
    Section_LibraryAuth = 0,    /* 位置測位ライブラリ認証用 */
    Section_Measureing          /* 測位計測 */
};

/* フィールド定数定義(位置測位ライブラリ認証用セクション) */
enum {
    Field_ApiKey = 0,   /* APIキー */
    Field_ClientId,     /* クライアントID */
    Field_Secret        /* 秘密鍵 */
};

/* フィールド定数定義(測位計測セクション) */
enum {
    Field_Accuracy = 0,                 /* 測位精度(m) */
    Field_Distance,                     /* 距離(m) */
    Field_BatteryLevInterval,           /* バッテリー閾値 */
    Field_SignificantChangeAccuracy,    /* 測位切替精度 */
    Field_LowEnergyModeSwitch,          /* 省電力モード */
    Field_MonitoringAreaSmallRadius,    /* 監視領域半径(小)(m) */
    Field_MonitoringAreaMiddleRadius,   /* 監視領域半径(中)(m) */
    Field_MonitoringAreaLargeRadius,    /* 監視領域半径(大)(m) */
    Field_MonitoringAreaOversizeRadius, /* 監視領域半径(特大)(m) */
    Field_MonitoringSlowSpeed,          /* 監視移動速度(低)(km/h) */
    Field_MonitoringMiddleSpeed,        /* 監視移動速度(中)(km/h) */
    Field_MonitoringHighSpeed           /* 監視移動速度(高)(km/h) */
};

@interface GFaLocationManagerSettingViewController ()

- (UITextField *)findTextField_:(NSInteger)section row:(NSInteger)row;
- (void)setValueToLocationManagerSetting_;
- (void)setValueFromLocationManagerSetting_;
- (NSNumber *)convertTextToNumber_:(NSString *)target;

@end

@implementation GFaLocationManagerSettingViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.title = @"位置測位・ライブラリ設定";

    [self setValueFromLocationManagerSetting_];
    [self setupAuthDevMenu_];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    
    [self setValueToLocationManagerSetting_];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/* 初期化ボタン押下 */
- (IBAction)pushDefaultButton:(id)sender
{
    // セクションタイトルの背景色をグレーに設定
    GFaLocationManagerSettingModel *settingModel = [GFaLocationManagerSettingModel defaultModel];
    [settingModel resetSetting];
    [settingModel saveToUserDefaults];
    [self setValueFromLocationManagerSetting_];
    [self setupAuthDevMenu_];
}

#pragma mark UITableViewDelegate

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];

    __block UITextField *textField = [self findTextField_:indexPath.section row:indexPath.row];

    if (textField != nil)
    {
        [GFaUtility showInputAlert:self title:textField.accessibilityLabel message:@"" text:textField.text cancelButtonTitle:@"キャンセル" otherButtonTitle:@"確定" keyboardType:textField.keyboardType otherHandler:^(NSString *value) {
            textField.text = value;
        }];
    }
}

#pragma mark Private

/* 入力フィールド取得 */
- (UITextField *)findTextField_:(NSInteger)section row:(NSInteger)row
{
    switch (section) {
        case Section_LibraryAuth:
            switch (row) {
                case Field_ApiKey:
                    return self.apikeyTextField;
                case Field_ClientId:
                    return self.clientIDTextField;
                case Field_Secret:
                    return self.secretTextField;
                default:
                    break;
            }
            break;
        case Section_Measureing:
            switch (row) {
                case Field_Accuracy:
                    return self.accuracyTextField;
                case Field_Distance:
                    return self.distanceTextField;
                case Field_BatteryLevInterval:
                    return self.batteryLevIntervalTextField;
                case Field_SignificantChangeAccuracy:
                    return self.significantChangeAccuracyTextField;
                case Field_LowEnergyModeSwitch:
                    return nil;
                case Field_MonitoringAreaSmallRadius:
                    return self.monitoringAreaSmallRadiusTextField;
                case Field_MonitoringAreaMiddleRadius:
                    return self.monitoringAreaMiddleRadiusTextField;
                case Field_MonitoringAreaLargeRadius:
                    return self.monitoringAreaLargeRadiusTextField;
                case Field_MonitoringAreaOversizeRadius:
                    return self.monitoringAreaOversizeRadiusTextField;
                case Field_MonitoringSlowSpeed:
                    return self.monitoringSlowSpeedTextField;
                case Field_MonitoringMiddleSpeed:
                    return self.monitoringMiddleSpeedTextField;
                case Field_MonitoringHighSpeed:
                    return self.monitoringHighSpeedTextField;
                default:
                    break;
            }
        default:
            break;
    }
    return nil;
}

/* 画面入力値を設定保持クラスに設定 */
- (void)setValueToLocationManagerSetting_
{
    GFaLocationManagerSettingModel *settingModel = [GFaLocationManagerSettingModel defaultModel];
    
    [settingModel setObject:self.apikeyTextField.text forKey:MeasuringSettingApiKey];
    [settingModel setObject:self.clientIDTextField.text  forKey:MeasuringSettingClientIDKey];
    [settingModel setObject:self.secretTextField.text  forKey:MeasuringSettingSecretKey];
    [settingModel setObject:[NSNumber numberWithInteger:self.authEnvPullDown.tag] forKey:MeasuringSettingAuthEnvKey];
    
    [settingModel setObject:[self convertTextToNumber_:self.accuracyTextField.text] forKey:MeasuringSettingAccuracyKey];
    [settingModel setObject:[self convertTextToNumber_:self.distanceTextField.text] forKey:MeasuringSettingDistanceKey];
    [settingModel setObject:[self convertTextToNumber_:self.batteryLevIntervalTextField.text] forKey:MeasuringSettingBatteryLevIntervalKey];
    [settingModel setObject:[self convertTextToNumber_:self.significantChangeAccuracyTextField.text] forKey:MeasuringSettingSignificantChangeAccuracyKey];
    
    [settingModel setObject:[self convertTextToNumber_:self.monitoringAreaSmallRadiusTextField.text] forKey:MeasuringSettingMonitoringAreaSmallRadiusKey];
    [settingModel setObject:[self convertTextToNumber_:self.monitoringAreaMiddleRadiusTextField.text] forKey:MeasuringSettingMonitoringAreaMiddleRadiusKey];
    [settingModel setObject:[self convertTextToNumber_:self.monitoringAreaLargeRadiusTextField.text] forKey:MeasuringSettingMonitoringAreaLargeRadiusKey];
    [settingModel setObject:[self convertTextToNumber_:self.monitoringAreaOversizeRadiusTextField.text] forKey:MeasuringSettingMonitoringAreaOversizeRadiusKey];
    [settingModel setObject:[self convertTextToNumber_:self.monitoringSlowSpeedTextField.text] forKey:MeasuringSettingMonitoringSlowSpeedKey];
    [settingModel setObject:[self convertTextToNumber_:self.monitoringMiddleSpeedTextField.text] forKey:MeasuringSettingMonitoringMiddleSpeedKey];
    [settingModel setObject:[self convertTextToNumber_:self.monitoringHighSpeedTextField.text] forKey:MeasuringSettingMonitoringHighSpeedKey];
    
    [settingModel setObject:@(self.lowEnergyModeSwitch.on) forKey:MeasuringSettingLowEnergyModeKey];
    
    [settingModel saveToUserDefaults];
}

/* 設定保持クラスの内容を画面入力値に設定 */
- (void)setValueFromLocationManagerSetting_
{
    GFaLocationManagerSettingModel *settingModel = [GFaLocationManagerSettingModel defaultModel];
    
    self.apikeyTextField.text = [settingModel objectForKey:MeasuringSettingApiKey];
    self.clientIDTextField.text = [settingModel objectForKey:MeasuringSettingClientIDKey];
    self.secretTextField.text = [settingModel objectForKey:MeasuringSettingSecretKey];
    self.authEnvPullDown.tag = [[settingModel objectForKey:MeasuringSettingAuthEnvKey] integerValue];
    
    self.accuracyTextField.text = [[settingModel objectForKey:MeasuringSettingAccuracyKey] stringValue];
    self.distanceTextField.text = [[settingModel objectForKey:MeasuringSettingDistanceKey] stringValue];
    self.batteryLevIntervalTextField.text = [[settingModel objectForKey:MeasuringSettingBatteryLevIntervalKey] stringValue];
    self.significantChangeAccuracyTextField.text = [[settingModel objectForKey:MeasuringSettingSignificantChangeAccuracyKey] stringValue];
    self.monitoringAreaSmallRadiusTextField.text = [[settingModel objectForKey:MeasuringSettingMonitoringAreaSmallRadiusKey] stringValue];
    self.monitoringAreaMiddleRadiusTextField.text = [[settingModel objectForKey:MeasuringSettingMonitoringAreaMiddleRadiusKey] stringValue];
    self.monitoringAreaLargeRadiusTextField.text = [[settingModel objectForKey:MeasuringSettingMonitoringAreaLargeRadiusKey] stringValue];
    self.monitoringAreaOversizeRadiusTextField.text = [[settingModel objectForKey:MeasuringSettingMonitoringAreaOversizeRadiusKey] stringValue];
    self.monitoringSlowSpeedTextField.text = [[settingModel objectForKey:MeasuringSettingMonitoringSlowSpeedKey] stringValue];
    self.monitoringMiddleSpeedTextField.text = [[settingModel objectForKey:MeasuringSettingMonitoringMiddleSpeedKey] stringValue];
    self.monitoringHighSpeedTextField.text = [[settingModel objectForKey:MeasuringSettingMonitoringHighSpeedKey] stringValue];
    
    self.lowEnergyModeSwitch.on = [[settingModel objectForKey:MeasuringSettingLowEnergyModeKey] boolValue];
}

/* 文字列を数値に変換 */
- (NSNumber *)convertTextToNumber_:(NSString *)target
{
    NSNumber *value = nil;
    
    if ( target.length > 0 )
    {
        value = @([target integerValue]);
    }
    
    return value;
}

/* 認証サーバ環境メニューを設定 */
- (void)setupAuthDevMenu_
{
    /* メニュー選択後Block */
    void (^selected)(int) = ^(int authEnv) {
        self.authEnvPullDown.tag = authEnv;
        [self.authEnvPullDown setTitle:[self getAuthEnvMenuTitle_:authEnv] forState:UIControlStateNormal];
    };
    
    /* メニュー選択時コールバックアクション設定 */
    NSMutableArray* actions = [[NSMutableArray alloc] init];
    [actions addObject:[UIAction actionWithTitle:kAuthEnvMenuTitleTest image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Test);
    }]];
    [actions addObject:[UIAction actionWithTitle:kAuthEnvMenuTitleStg image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Stg);
    }]];
    [actions addObject:[UIAction actionWithTitle:kAuthEnvMenuTitlePro image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Pro);
    }]];
    self.authEnvPullDown.menu = [UIMenu menuWithTitle:@"" children:actions];
    
    // 初期表示メニュータイトル設定
    [self.authEnvPullDown setTitle:[self getAuthEnvMenuTitle_:self.authEnvPullDown.tag] forState:UIControlStateNormal];
    self.authEnvPullDown.showsMenuAsPrimaryAction = true;
}

/* 認証サーバ環境メニュータイトルを取得 */
-(NSString *)getAuthEnvMenuTitle_:(NSInteger)authEnv
{
    switch (authEnv) {
        case AuthEnv_Test:
            return kAuthEnvMenuTitleTest;
        case AuthEnv_Stg:
            return kAuthEnvMenuTitleStg;
        case AuthEnv_Pro:
        default:
            return kAuthEnvMenuTitlePro;
    }
}

@end
