//
//  GFaDebug.h
//  For Application Develop
//
//  Copyright (c) 2013 ZENRIN DataCom CO., LTD. All Rights Reserved.
//
//

#ifndef ApplicationName_GFaDebug_h
#define ApplicationName_GFaDebug_h

#if defined(GFa_DEBUG)

  #define GFaAssert(x)                                                      \
    assert(x)
  
  #define GFaBreakIf(x)                                                     \
    if ((x)) {                                                              \
        NSLog(@"%s %d: (%s)\n", __FUNCTION__, __LINE__, #x);                \
        fflush(stderr);                                                     \
        break;                                                              \
    }

  #define GFaBreakQuietIf(x)                                                \
    if ((x)) { break; }

  #define GFaCheck(x)                                                       \
    if (!(x)) {                                                             \
        NSLog(@"%s %d: (%s) is NO.", __FUNCTION__, __LINE__, #x);           \
    }

  #define GFaCheckEqual(x, v)                                               \
    if ((x) != (v)) {                                                       \
        NSLog(@"%s %d: (%s) != (%s).", __FUNCTION__, __LINE__, #x, #v);     \
    }

  #define GFaLog(...)                                                       \
    do {                                                                    \
        NSLog(@"%s %d: ", __FUNCTION__, __LINE__);                          \
        NSLog(__VA_ARGS__);                                                 \
    } while (0)

  #define GFaLogRaw(...)                                                    \
    do {                                                                    \
        NSLog(__VA_ARGS__);                                                 \
    } while (0)

  #define GFaLogIf(flag)                                                    \
    if ((flag)) {                                                           \
        NSLog(@"%s %d: (%s)\n", __FUNCTION__, __LINE__, #flag);             \
    }

  #define GFaReturnIf(x)                                                    \
    if (x) {                                                                \
        NSLog(@"%s %d: (%s)\n", __FUNCTION__, __LINE__, #x);                \
        return;                                                             \
    }

  #define GFaReturnValueIf(x, value)                                        \
    if (x) {                                                                \
        NSLog(@"%s %d: (%s)\n", __FUNCTION__, __LINE__, #x);                \
        return value;                                                       \
    }

#else

  #define GFaAssert(x)               if (x) { }
  #define GFaBreakIf(x)              if ((x)) { break; }
  #define GFaBreakQuietIf(x)         if ((x)) { break; }
  #define GFaCheck(x)                (x)
  #define GFaCheckEqual(x, v)        (x)
  #define GFaLog(...)
  #define GFaLogIf(flag)             if ((flag)) { }
  #define GFaLogRaw(...)
  #define GFaReturnIf(x)             if (x) return;
  #define GFaReturnValueIf(x, v)     if (x) return v;

#endif

#endif
